//More information at: https://www.aeq-web.com/
//Main Software for LoRa-Weatherstation 2020 (V 1.0|03052020)

#include <SPI.h>
#include <LoRa.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>

String device_id = "YOUR-IDXYZ";

#define ss 18         //LoRa SS PIN
#define rst 14        //LoRa RST PIN
#define dio0 26       //LoRa DIO PIN

int ldrpin = 37;      //PIN for LDR input
int anopin = 12;      //PIN for anemometer input

int wind_m_sec = 3;   //Wind measure time (seconds)
int sleep_time = 180; //ESP sleep time (seconds)

float temp;
float pressure;
float humidity;
float sunlight;
float wind;
int wind_cnt;

Adafruit_BME280 bme;


void setup() {
  bme.begin(0x76);              //BME280 I2C-address
  SPI.begin(5, 19, 27, 18);     //Pins for LoRa SPI
  LoRa.setPins(ss, rst, dio0);  //Pins for LoRa
  LoRa.begin(868E6);            //Start LoRa @ 868 MHz (EU-Band)
  
  esp_sleep_enable_timer_wakeup(sleep_time * 1000000);
}

void loop() {
  get_sensor_data();
  send_lora_packet();
  esp_deep_sleep_start();
}

void get_sensor_data() {
  temp = bme.readTemperature();
  pressure = bme.readPressure() / 100;
  humidity = bme.readHumidity();

  int ldr_value = analogRead(ldrpin);
  sunlight = (ldr_value * 0.02442);

  meassure_wind();
}

void send_lora_packet() {
  LoRa.beginPacket();
  LoRa.print(device_id);
  LoRa.print(";");
  LoRa.print(temp);
  LoRa.print(";");
  LoRa.print(humidity);
  LoRa.print(";");
  LoRa.print(pressure);
  LoRa.print(";");
  LoRa.print(wind);
  LoRa.print(";");
  LoRa.print(sunlight);
  LoRa.print(";");
  LoRa.endPacket();
  delay(500);
}

void meassure_wind() {
  wind_cnt = 0;
  attachInterrupt(digitalPinToInterrupt(anopin), intrrupt_cnt, RISING);
  delay(wind_m_sec * 1000); //Time for measure counts
  detachInterrupt(digitalPinToInterrupt(anopin));
  wind = ((float)wind_cnt / (float)wind_m_sec * 2.4) / 2; //Convert counts & time to km/h
}

void intrrupt_cnt() {//On count detected
  wind_cnt++;//Add one count
}
